use core::fmt;

pub type EntityId = String;

pub trait Entity: fmt::Debug + PartialEq + Clone + Send + Sync {
    fn key(&self) -> &EntityId;
}

macro_rules! impl_entity_partial_eq {
    ($entity_name:ident) => {
        impl PartialEq for $entity_name {
            #[inline]
            fn eq(&self, other: &Self) -> bool {
                use crate::app_core::shared::lib::Entity as _;
                self.key() == other.key()
            }
        }
    };
}

macro_rules! impl_base_entity {
    ($entity_name:ident) => {
        impl_entity_partial_eq!($entity_name);

        impl crate::app_core::shared::lib::Entity for $entity_name {
            #[inline]
            fn key(&self) -> &crate::app_core::shared::lib::EntityId {
                &self.key
            }
        }
    };
}
